#!/usr/bin/env python
# -*- coding: utf8 -*-
'''
Copyright © 2009 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).
'''
__author__ = "Keith Noto <noto@cs.tufts.edu>";
__license__ = "GPL";
__copyright__ = "Copyright © 2009 Keith Noto";
VERSION = __revision__ = "1.0";

import sys,string,math,tempfile;
from util import *;		
import autotype; 

# reformat a matrix X so that each column is numeric.  Replace k-valued string
# (i.e. nominal) features with k binary features, replace missing values in
# continuous features with a mid-range value
def numericize(X):
	X = list(X)
	N = len(X[0])
	types = autotype.current_column_types(X);
	values = [];
	for column in range(N):
		if types[column] in [str]:
			values += [ filter(lambda x : x is not None, uniq(sorted(map(lambda x:x[column],X)))) ];
			types[column] = str;
		elif types[column] in [float,int]:
			min_v = min(filter(lambda x : x is not None, map(lambda x:x[column],X)));
			max_v = max(filter(lambda x : x is not None, map(lambda x:x[column],X)));
			values += [ (min_v+max_v)/2.0 ];
			types[column] = float;
		else:
			types[column] = None;
			sys.stderr.write("# Numericize:  Ignore column %i of type %s.\n" % (column,types[column]));
			pass;
	for x in X:
		n_x  = []; # new row
		for column in range(N):
			if types[column] == str:
				# add k columns
				for i in range(len(values[column])): 
					n_x += [ 1 if x[column]==values[column][i] else 0 ]; # add k new features (all zero if x[column] is missing)
				##To print feature conversions:
				##if x==X[0]:	sys.stderr.write("# Feature %i (nominal) -> %i binary features %s\n" % (column+1,len(values[column]),range(len(n_x)-len(values[column])+1, len(n_x)+1)))
			elif types[column]==float:
				n_x += [ x[column] if x[column] is not None else values[column] ]; 	# add mid-range value calculated above
				##if x==X[0]:	sys.stderr.write("# Feature %i (continuous) -> feature %i\n" % (column+1,len(n_x)))
			else:
				pass;
		yield n_x;

# scale numeric columns in place
def unitscale(X):
	X = list(X);
	types = autotype.current_column_types(X);
	ranges = {}
	for column in range(len(types)):
		if types[column] in [float,int]:
			min_v = float(min(filter(lambda v : v is not None, map(lambda x : x[column], X))));
			max_v = float(max(filter(lambda v : v is not None, map(lambda x : x[column], X))));
			if (min_v,max_v) == (0,1):
				pass; #done
			elif max_v==min_v:
				# const
				for i in range(len(X)):
					if (X[i][column] is not None):
						X[i][column] = 0;
			else:
				for i in range(len(X)): 
					if X[i][column] is not None:
						X[i][column] = (X[i][column] - min_v) / (max_v-min_v); 
	return X;


## read an array of numerical or string data into memory
def read_matrix_data(input, delim=None, comment='\n'):
	X = list( \
		map(lambda x : map(lambda v:v.strip(),x),  \
	    map(lambda L : L.split(delim), \
	    filter(lambda L : filter(lambda c : c not in string.whitespace, L),\
	    map(lambda L : L.split(comment,1)[0], \
	    map(lambda L : filter(lambda c : c in string.printable, L), \
	    input))))));
	autotype.cast(X, autotype.infer_column_types(X)); 
	return X;

# Combine, Apply function f which operates on a matrix, and re-break parallel matrices
def train_test_apply(trainset,testset,f):
	X = list(f(gcat((trainset,testset))))
	assert len(X) == len(trainset)+len(testset);
	trainset = X[:len(trainset)];
	testset  = X[len(trainset):]; 
	return (trainset,testset);
	
# Remove features from parallel matricies that have zero variance
def rm_const_features(X):
	types = autotype.current_column_types(X);
	N = len(types);
	keepers = [True] * N;
	for column in range(N):
		if types[column] is None:
			keepers[column] = False;
		else:
			min_v = min(filter(lambda v : v is not None, map(lambda x : x[column], X)));
			max_v = max(filter(lambda v : v is not None, map(lambda x : x[column], X)));
			if (min_v == max_v):
				sys.stderr.write("# Remove const feature %i\n" % (column+1));
				keepers[column] = False;
	return map(lambda x : map( lambda i : x[i], filter(lambda i : keepers[i], range(N)) ), X);
