# -*- coding: utf8 -*-

'''

Definitions (and utility functions?) for our anomaly detection approach.

Copyright © 2009 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).

'''
__author__ = "Keith Noto <noto@cs.tufts.edu>";
__license__ = "GPL";
__copyright__ = "Copyright © 2009 Keith Noto";
VERSION = __revision__ = "1.0";

import math, sys;
from util import gpairs,gmap;

# A classifier models a function f( x ) -> y and can generate predictions.  The
# following interface must be implemented:
class Classifier:
	# Predict some type of value from the test example x.
	def __call__(self,x):
		raise NotImplementedError();
	# Generate predictions for a SET of test vectors, X.
	# Override this method if it is more efficient to produce predictions en masse.
	def predictions(self, X):
		for x in X:
			yield self(x);	

# Mapping from an example vector to a (x,y) tuple where x is a training vector
# and y is a target value (rho, in the paper)
class LSet:
	def __init__(self, k, predictors):
		self.k,self.predictors = k,predictors;
	def __call__(self, x):
		return (map(lambda j : x[j], self.predictors), x[self.k]); # return tuple

# Map a training set to a parallel set of (observed,predicted) pairs.
def trainset_predict(trainset, sublearn, N):
	if N:	# cross-validation mode
		for g in crossval( trainset, N=N, f=lambda A,B : ttp(A,B,sublearn) ):
			for (obs,prd) in g:
				yield (obs,prd);
	else:	# generate straight-up training set predictions
		for (obs,prd) in ttp(trainset,trainset,sublearn):
			yield (obs,prd);
	
# Train/Test/Predict
# Generate a set of (observed,predicted) pairs for each example in B, based on the training set A.
def ttp(trainset, testset, sublearn):
	f = sublearn(trainset)
	for (y,g) in gpairs(gmap(lambda (x,y):y,testset), f.predictions(gmap(lambda (x,y):x, testset))):
		yield (y,g);

# Transpose list of lists.
def transpose(A):
	A = list(A);
	AT = [];
	for column in range( len(A[0]) ):
		AT += [map(lambda Arow:Arow[column],A)];
	return AT;

# N-fold cross-validation
def crossval( X, N, f ):
	X = list(X);
	for fold in range(N):
		sys.stderr.write("# fold %i\n" % (fold)) # notox
		trainset = map(lambda index:X[index], filter(lambda index : index%N!=fold, range(len(X))));
		testset  = map(lambda index:X[index], filter(lambda index : index%N==fold, range(len(X))));
		yield f(trainset,testset);

