#!/usr/bin/env python
# -*- coding: utf8 -*-
'''
Shared functions

Copyright © 2009 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).

'''
__author__ = "Keith Noto <noto@cs.tufts.edu>";
__license__ = "GPL";
__copyright__ = "Copyright © 2009 Keith Noto";
VERSION = __revision__ = "1.0";

import subprocess;

# Output stream that doesn't do anything.  Useful as a default or 'silent'
# output stream.
class NullStream:
	def readline(self,_):
		pass;
	def read(self,_):
		pass;
	def write(self, _):
		pass;
	def flush(self):
		pass;

# generate pairs from parallel lists (iterables)
def pairwise(A,B):
	bit = iter(B);
	for a in A:
		b = bit.next();
		yield (a,b);

# pairwise, but assume A,B are equal lengths
def gpairs(A,B):
	bit = iter(B)
	for a in A:
		try:
			b = bit.next()
			yield (a,b)
		except StopIteration:
			raise Exception("More A than B:  Asked to generate pairs from sets of unequal size.")
	try:
		b = bit.next()
		raise Exception("More B than A:  Asked to generate pairs from sets of unequal size.")
	except StopIteration:
		pass;

# generate index numbers while you iterate
# useful if you need to index into a parallel data structure, but you need to filter 'it'
def iwalk(it):
	i=0
	for x in it:
		yield (i,x)
		i+=1

# create a generator instead of a list
def gmap(f,it):
	for x in it:
		yield f(x)

# generate instead of creating a list in memory
def gfilter(f,it):
	for x in it:
		if f(x):
			yield x

# generate items in order from multiple iterators
def gcat(its):	 # "it's..."
	for it in its:
		for x in it:
			yield x

# compute length without necessarily creating a list in memory
def glen(it):
	len=0
	for x in it:
		len+=1
	return len

# generate unique members 
def uniq(X):
	m = {}
	for x in X:
		if x not in m:
			yield x
		m[x]=True

# return (count,unique member) pairs
def uniqc(X):
	m = {}
	for x in X:
		m[x] = m.get(x,0) + 1;
	return map(lambda (v,c):(c,v), m.iteritems()) # return (count,value) pairs
	
## generate (count, unique member) pairs from a sorted list 
## (can be slightly) faster than uniqc if list is already sorted, but not worth the trouble (notox)
#def guniqc(X):
#	(c,last) = (0,None);
#	for x in X:
#		if not c:
#			(c,last) = (1,x);
#		elif x!=last:
#			yield (c,last);
#			(c,last) = (1,x);
#		else:
#			c+=1;
#	if c:
#		yield (c,last);


## Run a process, return (stdout,stderr).  sink stream<=>finish stdout/stderr right away
import subprocess;
def proc(cmd, input=[], sink=None):
	p = subprocess.Popen([cmd], shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,close_fds=True); # shell b/c we want $PATH available
	map( lambda line : p.stdin.write(line), input ); # write all input to the process
	p.stdin.close();
	if sink:
		finish(p.stdout,sink)
		finish(p.stderr,sink)
		return (None,None);
	else:
		return (p.stdout,p.stderr); # these must be closed by the caller

# Close streams, e.g. returned from `proc'
# @param out Write data to this stream
def finish(stream, out=NullStream()):
	if stream is not None:
		byte = True;
		while byte:
			byte = stream.read(1);
			if out and byte:
				out.write(byte);
				out.flush();
		stream.close();

