#!/usr/bin/env python
# -*- coding: utf8 -*-
'''
LOF code

Copyright © 2009 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).

'''
__author__ = "Keith Noto <noto@cs.tufts.edu>";
__license__ = "GPL";
__copyright__ = "Copyright © 2009 Keith Noto";
VERSION = __revision__ = "1.0";

import sys,string,os, random;
from util import *;
from format import *;

# Return set of testset LOF scores, with None in place of test set examples
# replaced due to limiting of the test set size (the testset_limit arg)
def lof(trainset, testset, infilename, MinPtsLB, MinPtsUB, path="./LOF2", T=None, trials_todo=[]):

	trainset=list(trainset) 
	testset = list(testset)
	N = len(trainset[0])

	if T:
	
		R = random.Random()
		R.seed( 92353 )

		testset_scores = [ 0.0 ] * len(testset)
		for t in range(T):

			# Do the randomizing stuff for *all* trials

			F = range(N) 					# start with all features
			R.shuffle(F)				# mix 'em up
			f = R.randint(N/2, N-1)	# select this many features
			F = F[:f]						# take the first f of the mixed-up set
		
			# only actually execute given trials
			if t in trials_todo:
				sys.stderr.write("# Feature bagging (t=%i):  Select features %s\n" % (t, str(map(lambda i:i+1,F))))
				testset_scores_t = lof_feature_set(trainset,testset,infilename,MinPtsLB,MinPtsUB,path,F=F)
				assert len(testset_scores_t) == len(testset_scores) == len(testset)
				for i in range(len(testset_scores)):
					testset_scores[i] += testset_scores_t[i]

	else:
		testset_scores = lof_feature_set(trainset,testset,infilename,MinPtsLB,MinPtsUB,path,F=range(N))
	
	return testset_scores;

def lof_feature_set(trainset,testset,infilename,MinPtsLB,MinPtsUB,path,F):

	data = map( lambda example : map(lambda i : example[i], F), trainset+testset )
	infile = open(infilename,'w');
	infile.write( '\n'.join(map( lambda example : ','.join(map(str,example)), data)) )
	infile.close()

	cmd = "%s %i %i %i %i %i %s" % (path, MinPtsLB, MinPtsUB, len(trainset), len(testset), len(F), infilename)
	sys.stderr.write("# $ %s\n" % (cmd)); 
	(lof_scores, error) = proc(cmd);
	if error:
		sys.stderr.write(error.read()); 
		error.close()
	testset_scores = []
	for i in range(len(testset)):	# fill in score for all (possibly sampled) scores returned by LOF process
		scoreline = lof_scores.readline();
		if not scoreline:
			sys.stderr.write("Error running LOF:  Missing output for example(s) #%i and higher\n" % (i+1)); 
			break;
		else:
			testset_scores += [ float(scoreline) ]
	lof_scores.close();
	os.unlink(infilename);

	return testset_scores;


