#!/usr/bin/env python
# -*- coding: utf8 -*-
'''

Discrete probabilitiy distribution(s)

Copyright © 2010 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).
'''
__author__ = "Keith Noto <noto@cs.tufts.edu>";
__license__ = "GPL";
__copyright__ = "Copyright © 2009 Keith Noto";
VERSION = __revision__ = "1.0";

import sys,string,math
import pdf

# Only integer values 0,1,...,n-1 are applicable
# Can be used as a PDF but with care...
class Multinomial(pdf.PDF):	# could inherit from pdf
	def __init__(self,d):
		self.d = list(d);	# use 'd' the simple representation, list of probabilities
	def __str__(self):
		return "Multinomial PDF {%s}" % ( ','.join(map(lambda i : "%i:%g"%(i,self.d[i]), range(len(self.d)))))
	def normalize(self):
		Z = float(sum(self.d))
		self.d = map(lambda p : p/Z, self.d);	 # assume sum(d) > 0
	def p(self,i):
		if not ( 0 <= i < len(self.d) ):
			raise Exception("Illegal domain %s for multinomial(%s)" % (i, len(self.d)))
		return self.d[int(i)]
	def sample(self):
		mass = sum(self.d);
		for i in range(len(self.d)):
			if random.random() * mass < self.d[i]:
				return i
			mass -= self.d[i]
	def mean(self):
		return sum(map(lambda i : i * self.d[i], range(len(self.d)))) / sum(self.d)
	def median(self):
		t = sum(self.d)/2 # i.e. 0.5
		cd = 0	# cum. density (of the bins counted so far)
		for i in range(len(self.d)):
			if cd+self.d[i] >= t:
				return i + (t-cd)/self.d[i]
			cd += self.d[i]
	def mode(self):
		ans = 0
		for i in range(1,len(self.d)):
			if self.d[i] > self.d[ans]:
				ans = i
		return ans
	def CDF(self,i):
		return sum(map(lambda j : self.d[j], range(i+1)))
	def inverseCDF(self,p):
		Z = sum(self.d)
		t = p * Z;
		cd = 0
		for i in range(len(self.d)):
			cd += self.d[i];
			if cd >= t:
				return i
	def variance(self):
		mu = self.mean()
		xssd = 0
		for i in range(len(self.d)):
			xssd += self.d[i] * (mu-i) * (mu-i)
		return xssd 

# Smooth a multinomial (list of probabilities) Gaussian-shaped kernel
# This has the side-effect of making the distribution look more normal (i.e. the middle bins get more mass than the outer ones)
def smooth(A, sigma=1):
	n = len(A)	
	B = [0] * n
	for i in range(n):
		g = pdf.Gaussian(mu = i+0.5, sigma=sigma)
		mass = g.CDF(n) - g.CDF(0)
		for j in range(n):
			# transfer mass from A[i] to B[j]
			B[j] += A[i] * (g.CDF(j+1) - g.CDF(j)) / mass;
	return B

def surprisal(p):
	return -lg(p);

def entropy(D):
	if type(D)==float:	# beta distribution
		return entropy([D,1-D]); 
	elif type(D)==int:
		return entropy([float(D),1.0-D]);
	else:
		return sum(map(lambda p : 0 if p==0 else -p*lg(p), D)); 

def lg(x):
	return math.log(x) / 0.69314718055994528622676398299518041312694549560546875; # log(2)

