#include <stdio.h>
#include <stdlib.h>
#include <math.h> //sqrt
	
typedef float data;
typedef float dist;
typedef unsigned int uint;

dist distance(const data *x, const data *y, const uint N);
void distat (FILE *out, dist * const * const D, uint p0, uint p1);
void distat2(FILE *out, dist * const * const D, uint a0, uint a1, uint b0, uint b1);

int main(int argc, char **argv) {

	uint i,j,c;

	if (argc <= 1) { 
		fprintf(stderr,"Usage: %s N C_0 C_1 C_2 ... \n", argv[0]); 
		exit(-1);
	}

	uint N = atoi(argv[1]); 	// number of examples
	uint C = (uint)argc - 2;
	uint M[C+1];
	M[0] = 0;
	for (i=2; i<(uint)argc; i++) { 
		M[i-1] = M[i-2] + atoi(argv[i]);
		///	fprintf(stderr,"# M[%i] <- %i\n", i-1, M[i-1]);
	}

	/*


		 -| M[0] = 0
		  |
		  |
		  |
		 -| M[1] = start of anomaly class 1
		  |
		  |
		 -| M[2] = start of anomaly class 2
		 ...
		 -| M[n] = start of anomaly class n
		 -|__________________________


	*/

	// read data
	data **example = malloc(sizeof(data*) * M[C]); 
	for (i=0; i<M[C]; i++) { 

		example[i] = malloc(sizeof(data) * N); 

		for (c=0; c<N; c++) { 
			fscanf(stdin, "%f", example[i]+c); 
		}

		/// fprintf(stderr,"# Example i=%i is:  <", i); 
		/// for (c=0; c<N; c++) { fprintf(stderr," %f", example[i][c]); }
		/// fprintf(stderr," >\n");
		
	}

	dist **D;
	D = malloc( sizeof(dist*) * M[C] ); 
	for (i=0; i<M[C]; i++) { D[i] = malloc( sizeof(dist) * i ); }

	// precompute distance
	for (i=0; i<M[C]; i++) { 
		for (j=0; j<i; j++) { 
			D[i][j] = distance( example[i], example[j], N ); 
		}
	}

	for (i=0; i<M[C]; i++) { free(example[i]); }
	free(example);

	distat(stdout, D, M[0], M[C]); // all data
	fprintf(stdout, "\t");
	distat(stdout, D, M[1], M[C]); // all anomalies
	fprintf(stdout, "\t");
	distat2(stdout, D, M[0], M[1], M[1], M[C]); // norm <-> anomaly avg. distance
	
	for (c=0; c<C; c++) { 
		fprintf(stdout, "\t");
		distat(stdout, D, M[c], M[c+1]); 	// distance among this anomaly
		distat2(stdout, D, M[0], M[1], M[c], M[c+1]); 	// norm <-> this anomaly
	}

	fprintf(stdout, "\n");

	for (i=0; i<M[C]; i++) { free(D[i]); }
	free(D);

	return 0;

}//main

dist 
distance(const data *x, const data *y, const uint N) {

	uint c;
	dist ssd = 0.0;
	for (c=0; c<N; c++) { 
		dist diff = x[c] - y[c];
		ssd += diff*diff;
	}
	return sqrt(ssd);
}

void
distat(FILE *out, dist * const * const D, uint p0, uint p1) { 
	
	uint i,j;

	dist sum = 0.0;
	for (i=p0; i<p1; i++) { 
	for (j=p0; j<i; j++) { 
		sum += D[i][j]; 
	}}

	const uint n = ((p1-p0) * ((p1-p0)-1)) / 2;
	const dist mu = sum / n;

	dist ssd = 0.0;
	for (i=p0; i<p1; i++) { 
	for (j=p0; j<i; j++) { 
		dist diff = mu - D[i][j]; 
		ssd += diff*diff;
	}}

	/// fprintf(stderr, "# sd <- sqrt( ssd / n-1 ) <- sqrt( %g / %i-1 ) <- sqrt(%g) <- %g\n", ssd, n, ssd/(n-1), sqrt(ssd/(n-1)));

	const dist sd = sqrt( ssd / (n-1) ); // N-1 <=> Bessel's correction
	fprintf(out, "%15.10g\t%15.10g\n", mu, sd);

}


void 
distat2(FILE *out, dist * const * const D, uint a0, uint a1, uint b0, uint b1) { 
	uint i,j;
	dist sum = 0.0; 
	for (i=a0; i<a1; i++) { 
	for (j=b0; j<b1; j++) { 
		sum += D[j][i]; 
	}}

	const uint n = (a1-a0) * (b1-b0);
	const dist mu = sum / n;

	dist ssd = 0.0;
	for (i=a0; i<a1; i++) { 
	for (j=b0; j<b1; j++) { 
		dist diff = mu - D[i][j]; 
		ssd += diff*diff;
	}}

	const dist sd = sqrt( ssd / (n-1) ); // N-1 <=> Bessel's correction
	fprintf(out, "%15.10g\t%15.10g\n", mu, sd);

}


