#!/usr/bin/env bash

# Run frank.suite a bunch of times (one per underlying classifier, per feature)
# to compute the normalized surprisal per feature individually

dataset=$1
resultsdir=cluster.uci/results/$dataset

tmp=/tmp/frank.suite.$$.tmp
mkdir -p $tmp 

for ptype in tree linearsvm rbfsvm
do
	

	/bin/cp -r $resultsdir/labels $tmp;
	N=`cat $resultsdir/frac.$ptype | wc -l | gawk '{print $1-1}'`;
	let F=$N+1;
	for feature in `enum 1 ${F}`;
	do

		let L=$feature+1;
		cat $resultsdir/frac.$ptype | head -1 > $tmp/frac.$feature.$ptype 
		cat $resultsdir/frac.$ptype | lines $L >> $tmp/frac.$feature.$ptype
		
		code/scripts/frank.suite $tmp/frac.$feature.$ptype 2> /dev/null;

		echo -ne "$dataset\t$ptype\t$feature\t";

		for ext in normalized-surprisal normdist-surprisal; 
		do
			echo -ne "`paste $resultsdir/labels $tmp/frac.$feature.$ptype.$ext | gawk '{print $3,$4}' | roc | auc`\t";
			echo -ne "`cat $tmp/frac.$feature.$ptype.$ext | ag --mean`\t";
			echo -ne "`cat $tmp/frac.$feature.$ptype.$ext | ag --stdev`\t";
		done;

		echo -ne "\n";



	done
		

done;

/bin/rm -rf $tmp;
