#!/usr/bin/env python
# -*- coding: utf8 -*-
'''
Copyright © 2010 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).

'''
__author__ = "Keith Noto <noto@cs.tufts.edu>";
__license__ = "GPL";
__copyright__ = "Copyright © 2009 Keith Noto";
VERSION = __revision__ = "1.0";

import sys,math;
import pdf;

inf = float('inf')

class SumOfGaussians(pdf.Mixture):
	def __init__(self, sample, sigma):
		S = list(sample);
		w = 1.0 / len(S)
		self.mixture = dict(map( lambda mu : (pdf.Gaussian(mu, sigma), w), S ))
	def pb(self,x,min,max):
		if x < min or x > max:
			return 0.0;
		fx = 0
		for (g,w) in self.mixture.iteritems():
			fx += w * g.p(x) / (g.CDF(max) - g.CDF(min))
		return fx
	def entropy(self,min,max):
		root2 = math.sqrt(2)
		root2pi = math.sqrt(2*math.pi)
		log2pi = math.log(2*math.pi)
		ent = 0.0;
		for (g,w) in self.mixture.iteritems():
			m = mu = g.mu
			s = sigma = g.sigma
			r = g.CDF(max)-g.CDF(min);
			for (x,f) in [ (max,1), (min,-1) ]:
				try:
					eterm = math.exp((m-x)*(m-x)/(2*s*s));
				except OverflowError:
					eterm = inf;
				denom = 4 * eterm * root2pi * s
				addend = 2 * (-2*m / denom + 2*x / denom + (root2pi * s * pdf.erf((m - x)/(root2*s)) * (1 + log2pi - 2*math.log(1/s))) / (4 * root2pi * s))
				ent += w*f*addend/r;
		return -ent / math.log(2) # convert to bits
	def surprisal(self,x,min=-inf,max=inf):
		fx = self.pb(x,min,max)
		return -lg(fx) if fx>0 else inf;

def lg(x):
    return math.log(x) / 0.69314718055994528622676398299518041312694549560546875; # log(2)
